/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: InstallationServiceMBean.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.container.lifecycle;

import javax.jbi.JBIException;
import javax.jbi.component.ServiceUnitManager;

import org.ow2.petals.container.event.StateChangeFailedEvent;
import org.ow2.petals.container.event.StateChangedEvent;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.util.LoggingUtil;

/**
 * This class manage the life cycle of deployed service units
 * 
 * @author Olivier Fabre - EBM Websourcing
 * 
 */
public class ServiceUnitLifeCycle extends LifeCycleAbstract {

    private AdminServiceMBean adminService;

    private ServiceUnit serviceUnit;

    private String serviceUnitRootPath;

    private ServiceUnitManager serviceUnitManager;

    public ServiceUnitLifeCycle(ServiceUnit serviceUnit, AdminServiceMBean managerService,
            String serviceUnitRootPath, LoggingUtil log) {
        super();
        super.init(null, log);

        this.adminService = managerService;
        this.serviceUnit = serviceUnit;
        this.serviceUnitRootPath = serviceUnitRootPath;
        this.name = serviceUnit.getIdentification().getName();
    }

    /**
     * Returns the ServiceUnitManager associated with the service unit's
     * targeted component
     * 
     * @return null if there's no Service Unit Manager
     */
    public ServiceUnitManager getServiceUnitManager() {
        if (this.serviceUnitManager == null) {
            ComponentLifeCycle clc = adminService.getComponentLifeCycleByName(serviceUnit
                    .getTarget().getComponentName());

            if (clc != null) {
                this.serviceUnitManager = clc.getComponent().getServiceUnitManager();
            }
        }

        return this.serviceUnitManager;
    }

    /**
     * Return the name of the targeted component.
     * 
     * @return the name of the targeted component
     */
    public String getTargetComponentName() {
        return serviceUnit.getTarget().getComponentName();
    }

    public String getServiceUnitRootPath() {
        return serviceUnitRootPath;
    }

    public String getSuName() {
        return name;
    }

    /**
     * Do nothing because there's no Service Unit MBean registered in JMX !
     */
    @Override
    protected void stateChanged(StateChangedEvent event) {
        // do nothing
    }

    /**
     * Do nothing because there's no Service Unit MBean registered in JMX !
     */
    @Override
    public void stateChangeFailed(StateChangeFailedEvent event) {
        // do nothing
    }

    @Override
    public void doInit() throws JBIException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().init(name, serviceUnitRootPath);
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }

    }

    @Override
    public void doStart() throws JBIException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().start(name);
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }
    }

    @Override
    public void doStop() throws JBIException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().stop(name);
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }
    }

    @Override
    public void doShutdown() throws JBIException {
        // Set the context class loader to the component's one just for the
        // method invocation
        try {
            Thread.currentThread().setContextClassLoader(
                    getServiceUnitManager().getClass().getClassLoader());
            getServiceUnitManager().shutDown(name);
        } finally {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }
    }

    @Override
    protected String getJBIArtifactType() {
        return "Service Unit";
    }

}
