/**
 * PETALS: PETALS Services Platform Copyright (C) 2005 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: InstallDirectoryScanner.java,v 1.2 2006/04/07 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.autoload;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.TimerTask;

import org.ow2.petals.util.NioUtil;

/**
 * Periodically check if new Install or Deploy elements are presents in the
 * autoload directories.
 * 
 * @author alouis
 * 
 */
public class InstallDirectoryScanner extends TimerTask {

    private final File installDir;

    private final AutoLoaderServiceImpl autoLoader;

    private final ArchiveFileFilter archiveFilter;

    public InstallDirectoryScanner(AutoLoaderServiceImpl autoLoader, File installDir) {
        this.installDir = installDir;
        this.autoLoader = autoLoader;
        this.archiveFilter = ArchiveFileFilter.getInstance();
    }

    @Override
    public void run() {
        final File[] newInstall = this.installDir.listFiles(this.archiveFilter);
        final List<File> newInstallList = new ArrayList<File>(newInstall.length);

        if (newInstall != null && newInstall.length > 0) {
            for (final File newFile : newInstall) {
                if (NioUtil.checkAccess(newFile)) {
                    newInstallList.add(newFile);
                }
            }
            this.autoLoader.install(newInstallList);
        }
    }

}
