/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.task;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.HashMap;

import org.apache.commons.io.FileUtils;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task copies archive file to <i>installed</i> dir <br/>
 * Inputs : archive {@link URI} <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class CopyArchiveToInstalledDirTask extends AbstractFileManipulationTask {

    public CopyArchiveToInstalledDirTask(LoggingUtil log, String petalsInstallDirectory) {
        super(log, petalsInstallDirectory);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        URI archiveURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);
        File file = getArchiveFile(archiveURI);
        File installedArchive = getInstalledArchive(file);
        FileUtils.copyFile(file, installedArchive);
        context.put(TaskContextConstants.ARCHIVE_URI, installedArchive.toURI());
        FileUtils.forceDelete(file);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        if (context.containsKey(TaskContextConstants.ARCHIVE_URI)) {
            URI archiveURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);
            if (archiveURI != null) {
                File file = getArchiveFile(archiveURI);
                if (file != null) {
                    File installedArchive = this.getInstalledArchive(file);
                    if (installedArchive != null) {
                        try {
                            FileUtils.forceDelete(installedArchive);
                        } catch (IOException e) {
                            log.error("Component archive can't be deleted", e);
                        }

                    }
                }
            }
        }
    }

}
