/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.messaging.routing.module;

import java.util.Map;

import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.communication.topology.TopologyService;
import org.ow2.petals.jbi.component.context.ComponentContextImpl;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.jbi.messaging.routing.RouterServiceImpl;
import org.ow2.petals.jbi.messaging.routing.RoutingException;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.transport.Transporter;
import org.ow2.petals.util.LoggingUtil;

/**
 * A router module definition.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 2.2
 * 
 */
public abstract class RouterModule implements Module {

    protected RouterServiceImpl routerService;

    protected Transporter transporter;

    protected ConfigurationService configurationService;

    protected TopologyService topologyService;

    protected EndpointRegistry endpointService;

    protected Logger logger;

    protected LoggingUtil log;

    private boolean active = false;

    private boolean engaged = false;

    private Status status = Status.Unknown;

    protected Map<String, String> initArgs;

    /**
     * Creates a new instance of RouterModule
     */
    public RouterModule() {
    }

    /**
     * 
     * @param routerService
     */
    public final void init(RouterServiceImpl routerService) {
        this.routerService = routerService;
        this.transporter = routerService.getTransporter();
        this.configurationService = routerService.getConfigurationService();
        this.topologyService = routerService.getTopologyService();
        this.endpointService = routerService.getEndpointService();
        this.logger = routerService.getLogger();
        this.log = routerService.getLog();
    }

    /**
     * 
     * @return
     */
    public String getName() {
        return this.getClass().getName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#addComponent(org.ow2.petals.jbi.component.context.ComponentContextImpl)
     */
    public void addComponent(final ComponentContextImpl componentContext) throws RoutingException {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#receive(org.ow2.petals.jbi.component.context.ComponentContextImpl,
     *      long)
     */
    public MessageExchangeImpl receive(final ComponentContextImpl source, final long timeoutMS)
            throws RoutingException {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#removeComponent(org.ow2.petals.jbi.component.context.ComponentContextImpl)
     */
    public void removeComponent(final ComponentContextImpl componentContext)
            throws RoutingException {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#send(org.ow2.petals.jbi.component.context.ComponentContextImpl,
     *      org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl, long)
     */
    public void send(final ComponentContextImpl source, final MessageExchangeImpl exchange,
            final long timeOut) throws RoutingException {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.Router#sendSync(org.ow2.petals.jbi.component.context.ComponentContextImpl,
     *      org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl, long)
     */
    public MessageExchangeImpl sendSync(final ComponentContextImpl source,
            final MessageExchangeImpl exchange, final long timeOut) throws RoutingException {
        return null;
    }

    /**
     * Set module active status
     * 
     * @param active
     */
    public void setActive(boolean active) throws RouterModuleException {
        this.active = active;
    }

    /**
     * Get the module active status
     * 
     * @return
     */
    public boolean isActive() {
        return this.active;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.module.LifeCycle#init(java.util.Map)
     */
    public void init(Map<String, String> args) throws LifeCycleException {
        if (!getStatus().equals(Status.Unknown)) {
            throw new LifeCycleException("Module can not be initialized (current one is "
                    + getStatus() + ")");
        }
        try {
            this.initArgs = args;
            doInit();
            status = Status.Initialized;
        } catch (Exception e) {
            throw new LifeCycleException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.module.LifeCycle#start()
     */
    public final void start() throws LifeCycleException {
        if (!getStatus().equals(Status.Initialized)) {
            throw new LifeCycleException(
                    "Module can not be started since status is not 'initialized' (current one is "
                            + getStatus() + ")");
        }
        try {
            doStart();
            status = Status.Started;
        } catch (Exception e) {
            throw new LifeCycleException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.module.LifeCycle#stop()
     */
    public final void stop() throws LifeCycleException {
        if (!getStatus().equals(Status.Started)) {
            throw new LifeCycleException(
                    "Module can not be stopped since status is not 'started' (current one is "
                            + getStatus() + ")");
        }
        try {
            doStop();
            status = Status.Stopped;
        } catch (Exception e) {
            throw new LifeCycleException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.module.LifeCycle#getStatus()
     */
    public Status getStatus() {
        return this.status;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.module.Module#isEngaged()
     */
    public boolean isEngaged() {
        return engaged;
    }

    /**
     * 
     * @throws RoutingException
     */
    protected abstract void doInit() throws Exception;

    /**
     * 
     * @throws RoutingException
     */
    protected abstract void doStart() throws Exception;

    /**
     * 
     * @throws RoutingException
     */
    protected abstract void doStop() throws Exception;
}
