/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.security;

import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Security context
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 2.2
 * 
 */
public abstract class SecurityContext {

    private String userName;

    public abstract Set<?> getPrincipals();

    final ConcurrentHashMap<String, String> authorizedOperations = new ConcurrentHashMap<String, String>();

    /**
     * Creates a new instance of SecurityContext
     * 
     * @param userName
     */
    public SecurityContext(String userName) {
        this.userName = userName;
    }

    /**
     * @param allowedPrincipals
     * @return
     */
    public boolean isInOneOf(Set<?> allowedPrincipals) {
        HashSet<?> set = new HashSet<Object>(getPrincipals());
        set.retainAll(allowedPrincipals);
        return set.size() > 0;
    }

    /**
     * 
     * @return
     */
    public String getUserName() {
        return userName;
    }

    /**
     * 
     * @return
     */
    public ConcurrentHashMap<String, String> getAuthorizedOperations() {
        return authorizedOperations;
    }
}
