/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.kernel.interceptors.stat;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Timer;

import org.objectweb.fractal.julia.loader.Initializable;
import org.objectweb.fractal.julia.loader.Tree;

/**
 * This class provides an access to the statistic of methods class.
 *
 * @author Nicolas SALATGE - eBMWebSourcing
 *
 */
public class StatControllerImpl implements Initializable, StatController {

    /**
     * Default period.
     */
    private static final int DEFAULT_PERIOD = 3000;

    /**
     * contains the stats of the class.
     */
    private Set<InfoStat> stats = null;

    /**
     * Default constructor.
     */
    public StatControllerImpl() {
        this.stats = new HashSet<InfoStat>();
    }

    /**
     * implementation of the Initializable interface.
     * @param args the default argument
     */
    public void initialize(final Tree args) {
        String s = null;
        int period = StatControllerImpl.DEFAULT_PERIOD;
        if (args.getSubTree(0) != null) {
            s = args.getSubTree(0).toString();
        }

        try {
            period = Integer.parseInt(s);
        } catch (final NumberFormatException e) {
        }

        final Timer timer = new Timer();
        timer.schedule(new WriteStatsTask(this), 0, period);
    }

    /**
     * methods called by the associated interceptor.
     * @param method the method name
     * @return the begin time
     */
    public String preMethod(final String method) {
        synchronized (this) {
            final long begin = System.nanoTime();
            final DateFormat df = new SimpleDateFormat("[yyyy-MM-dd HH:mm:ss:SS]");
            final String beginDate = df.format(new Date());
            return begin + "_" + beginDate;
        }
    }

    /**
     * methods called by the associated interceptor.
     * @param method the method name
     * @param start the begin time
     */
    public void postMethod(final String method, final String start) {
        synchronized (this) {
            final StringTokenizer st = new StringTokenizer(start, "_");
            long begin = 0;
            String beginDate = "";
            if (st.countTokens() >= 2) {
                begin = Long.parseLong(st.nextToken());
                beginDate = st.nextToken();
            }
            final long time = System.nanoTime() - begin;
            this.stats.add(new InfoStat(method,
                        Thread.currentThread().getName(), begin, beginDate, time));

        }
    }

    public Set<InfoStat> getStats() {
        return this.stats;
    }

}
